# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc, 2017 Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




from .fetchers import NUMetadatasFetcher


from .fetchers import NUGlobalMetadatasFetcher

from bambou import NURESTObject


class NUGNMISession(NURESTObject):
    """ Represents a GNMISession in the VSD

        Notes:
            Represents GNMI session between gateway and Config Manager, This can only be created by netconfmgr user
    """

    __rest_name__ = "gnmisession"
    __resource_name__ = "gnmisessions"

    
    ## Constants
    
    CONST_SUBSCRIPTION_STATE_IN_PROGRESS = "IN_PROGRESS"
    
    CONST_STATUS_CONNECTED = "CONNECTED"
    
    CONST_ENTITY_SCOPE_GLOBAL = "GLOBAL"
    
    CONST_GATEWAY_VENDOR_NOKIA = "NOKIA"
    
    CONST_SUBSCRIPTION_STATE_INIT = "INIT"
    
    CONST_GATEWAY_VENDOR_CISCO = "CISCO"
    
    CONST_ENTITY_SCOPE_ENTERPRISE = "ENTERPRISE"
    
    CONST_SUBSCRIPTION_STATE_COMPLETED = "COMPLETED"
    
    CONST_SUBSCRIPTION_STATE_FAILED = "FAILED"
    
    CONST_STATUS_DISCONNECTED = "DISCONNECTED"
    
    

    def __init__(self, **kwargs):
        """ Initializes a GNMISession instance

            Notes:
                You can specify all parameters while calling this methods.
                A special argument named `data` will enable you to load the
                object from a Python dictionary

            Examples:
                >>> gnmisession = NUGNMISession(id=u'xxxx-xxx-xxx-xxx', name=u'GNMISession')
                >>> gnmisession = NUGNMISession(data=my_dict)
        """

        super(NUGNMISession, self).__init__()

        # Read/Write Attributes
        
        self._last_updated_by = None
        self._last_updated_date = None
        self._gateway_model = None
        self._gateway_vendor = None
        self._gateway_version = None
        self._embedded_metadata = None
        self._entity_scope = None
        self._creation_date = None
        self._associated_gateway_id = None
        self._associated_gateway_name = None
        self._status = None
        self._subscription_error = None
        self._subscription_failure_reason = None
        self._subscription_failure_retry_count = None
        self._subscription_state = None
        self._owner = None
        self._external_id = None
        
        self.expose_attribute(local_name="last_updated_by", remote_name="lastUpdatedBy", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="last_updated_date", remote_name="lastUpdatedDate", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="gateway_model", remote_name="gatewayModel", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="gateway_vendor", remote_name="gatewayVendor", attribute_type=str, is_required=False, is_unique=False, choices=[u'CISCO', u'NOKIA'])
        self.expose_attribute(local_name="gateway_version", remote_name="gatewayVersion", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="embedded_metadata", remote_name="embeddedMetadata", attribute_type=list, is_required=False, is_unique=False)
        self.expose_attribute(local_name="entity_scope", remote_name="entityScope", attribute_type=str, is_required=False, is_unique=False, choices=[u'ENTERPRISE', u'GLOBAL'])
        self.expose_attribute(local_name="creation_date", remote_name="creationDate", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_gateway_id", remote_name="associatedGatewayID", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="associated_gateway_name", remote_name="associatedGatewayName", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="status", remote_name="status", attribute_type=str, is_required=False, is_unique=False, choices=[u'CONNECTED', u'DISCONNECTED'])
        self.expose_attribute(local_name="subscription_error", remote_name="subscriptionError", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="subscription_failure_reason", remote_name="subscriptionFailureReason", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="subscription_failure_retry_count", remote_name="subscriptionFailureRetryCount", attribute_type=int, is_required=False, is_unique=False)
        self.expose_attribute(local_name="subscription_state", remote_name="subscriptionState", attribute_type=str, is_required=False, is_unique=False, choices=[u'COMPLETED', u'FAILED', u'IN_PROGRESS', u'INIT'])
        self.expose_attribute(local_name="owner", remote_name="owner", attribute_type=str, is_required=False, is_unique=False)
        self.expose_attribute(local_name="external_id", remote_name="externalID", attribute_type=str, is_required=False, is_unique=True)
        

        # Fetchers
        
        
        self.metadatas = NUMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        
        
        self.global_metadatas = NUGlobalMetadatasFetcher.fetcher_with_object(parent_object=self, relationship="child")
        

        self._compute_args(**kwargs)

    # Properties
    
    @property
    def last_updated_by(self):
        """ Get last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, value):
        """ Set last_updated_by value.

            Notes:
                ID of the user who last updated the object.

                
                This attribute is named `lastUpdatedBy` in VSD API.
                
        """
        self._last_updated_by = value

    
    @property
    def last_updated_date(self):
        """ Get last_updated_date value.

            Notes:
                Time stamp when this object was last updated.

                
                This attribute is named `lastUpdatedDate` in VSD API.
                
        """
        return self._last_updated_date

    @last_updated_date.setter
    def last_updated_date(self, value):
        """ Set last_updated_date value.

            Notes:
                Time stamp when this object was last updated.

                
                This attribute is named `lastUpdatedDate` in VSD API.
                
        """
        self._last_updated_date = value

    
    @property
    def gateway_model(self):
        """ Get gateway_model value.

            Notes:
                The model string of the gateway to which this session connected from Config Manager

                
                This attribute is named `gatewayModel` in VSD API.
                
        """
        return self._gateway_model

    @gateway_model.setter
    def gateway_model(self, value):
        """ Set gateway_model value.

            Notes:
                The model string of the gateway to which this session connected from Config Manager

                
                This attribute is named `gatewayModel` in VSD API.
                
        """
        self._gateway_model = value

    
    @property
    def gateway_vendor(self):
        """ Get gateway_vendor value.

            Notes:
                Vendor of the gateway to which this session connected from Config Manager

                
                This attribute is named `gatewayVendor` in VSD API.
                
        """
        return self._gateway_vendor

    @gateway_vendor.setter
    def gateway_vendor(self, value):
        """ Set gateway_vendor value.

            Notes:
                Vendor of the gateway to which this session connected from Config Manager

                
                This attribute is named `gatewayVendor` in VSD API.
                
        """
        self._gateway_vendor = value

    
    @property
    def gateway_version(self):
        """ Get gateway_version value.

            Notes:
                Boot image version of gateway to which this session connected from Config Manager

                
                This attribute is named `gatewayVersion` in VSD API.
                
        """
        return self._gateway_version

    @gateway_version.setter
    def gateway_version(self, value):
        """ Set gateway_version value.

            Notes:
                Boot image version of gateway to which this session connected from Config Manager

                
                This attribute is named `gatewayVersion` in VSD API.
                
        """
        self._gateway_version = value

    
    @property
    def embedded_metadata(self):
        """ Get embedded_metadata value.

            Notes:
                Metadata objects associated with this entity. This will contain a list of Metadata objects if the API request is made using the special flag to enable the embedded Metadata feature. Only a maximum of Metadata objects is returned based on the value set in the system configuration.

                
                This attribute is named `embeddedMetadata` in VSD API.
                
        """
        return self._embedded_metadata

    @embedded_metadata.setter
    def embedded_metadata(self, value):
        """ Set embedded_metadata value.

            Notes:
                Metadata objects associated with this entity. This will contain a list of Metadata objects if the API request is made using the special flag to enable the embedded Metadata feature. Only a maximum of Metadata objects is returned based on the value set in the system configuration.

                
                This attribute is named `embeddedMetadata` in VSD API.
                
        """
        self._embedded_metadata = value

    
    @property
    def entity_scope(self):
        """ Get entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        return self._entity_scope

    @entity_scope.setter
    def entity_scope(self, value):
        """ Set entity_scope value.

            Notes:
                Specify if scope of entity is Data center or Enterprise level

                
                This attribute is named `entityScope` in VSD API.
                
        """
        self._entity_scope = value

    
    @property
    def creation_date(self):
        """ Get creation_date value.

            Notes:
                Time stamp when this object was created.

                
                This attribute is named `creationDate` in VSD API.
                
        """
        return self._creation_date

    @creation_date.setter
    def creation_date(self, value):
        """ Set creation_date value.

            Notes:
                Time stamp when this object was created.

                
                This attribute is named `creationDate` in VSD API.
                
        """
        self._creation_date = value

    
    @property
    def associated_gateway_id(self):
        """ Get associated_gateway_id value.

            Notes:
                UUID of the gateway associated to this GNMI session.

                
                This attribute is named `associatedGatewayID` in VSD API.
                
        """
        return self._associated_gateway_id

    @associated_gateway_id.setter
    def associated_gateway_id(self, value):
        """ Set associated_gateway_id value.

            Notes:
                UUID of the gateway associated to this GNMI session.

                
                This attribute is named `associatedGatewayID` in VSD API.
                
        """
        self._associated_gateway_id = value

    
    @property
    def associated_gateway_name(self):
        """ Get associated_gateway_name value.

            Notes:
                Name of the gateway associated to this GNMI session.

                
                This attribute is named `associatedGatewayName` in VSD API.
                
        """
        return self._associated_gateway_name

    @associated_gateway_name.setter
    def associated_gateway_name(self, value):
        """ Set associated_gateway_name value.

            Notes:
                Name of the gateway associated to this GNMI session.

                
                This attribute is named `associatedGatewayName` in VSD API.
                
        """
        self._associated_gateway_name = value

    
    @property
    def status(self):
        """ Get status value.

            Notes:
                The status of the GNMI session to a gateway.

                
        """
        return self._status

    @status.setter
    def status(self, value):
        """ Set status value.

            Notes:
                The status of the GNMI session to a gateway.

                
        """
        self._status = value

    
    @property
    def subscription_error(self):
        """ Get subscription_error value.

            Notes:
                Last subscription error reported

                
                This attribute is named `subscriptionError` in VSD API.
                
        """
        return self._subscription_error

    @subscription_error.setter
    def subscription_error(self, value):
        """ Set subscription_error value.

            Notes:
                Last subscription error reported

                
                This attribute is named `subscriptionError` in VSD API.
                
        """
        self._subscription_error = value

    
    @property
    def subscription_failure_reason(self):
        """ Get subscription_failure_reason value.

            Notes:
                Detailed subscription error message

                
                This attribute is named `subscriptionFailureReason` in VSD API.
                
        """
        return self._subscription_failure_reason

    @subscription_failure_reason.setter
    def subscription_failure_reason(self, value):
        """ Set subscription_failure_reason value.

            Notes:
                Detailed subscription error message

                
                This attribute is named `subscriptionFailureReason` in VSD API.
                
        """
        self._subscription_failure_reason = value

    
    @property
    def subscription_failure_retry_count(self):
        """ Get subscription_failure_retry_count value.

            Notes:
                Number of times subscription has been attempted since first failure

                
                This attribute is named `subscriptionFailureRetryCount` in VSD API.
                
        """
        return self._subscription_failure_retry_count

    @subscription_failure_retry_count.setter
    def subscription_failure_retry_count(self, value):
        """ Set subscription_failure_retry_count value.

            Notes:
                Number of times subscription has been attempted since first failure

                
                This attribute is named `subscriptionFailureRetryCount` in VSD API.
                
        """
        self._subscription_failure_retry_count = value

    
    @property
    def subscription_state(self):
        """ Get subscription_state value.

            Notes:
                Status of gnmi subscriptions to device from Netconf Manager

                
                This attribute is named `subscriptionState` in VSD API.
                
        """
        return self._subscription_state

    @subscription_state.setter
    def subscription_state(self, value):
        """ Set subscription_state value.

            Notes:
                Status of gnmi subscriptions to device from Netconf Manager

                
                This attribute is named `subscriptionState` in VSD API.
                
        """
        self._subscription_state = value

    
    @property
    def owner(self):
        """ Get owner value.

            Notes:
                Identifies the user that has created this object.

                
        """
        return self._owner

    @owner.setter
    def owner(self, value):
        """ Set owner value.

            Notes:
                Identifies the user that has created this object.

                
        """
        self._owner = value

    
    @property
    def external_id(self):
        """ Get external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        return self._external_id

    @external_id.setter
    def external_id(self, value):
        """ Set external_id value.

            Notes:
                External object ID. Used for integration with third party systems

                
                This attribute is named `externalID` in VSD API.
                
        """
        self._external_id = value

    

    